/*************************************************************************
 * This file is part of Crosroads 1.76, a load balancer and fail over
 * utility for TCP. Copyright (c) Karel Kubat, distributed under GPL.
 * Visit http://crossroads.e-tunity.com for information.
 *************************************************************************/

#include "../crossroads.h"

typedef struct {
    char *arg;				/* cmdline argument */
    int narg;				/* required remaining args */
    int needconf;			/* configuration file required? */
    int (*handler)(int ac, char **av);	/* action handler */
} Handler;

int main (int argc, char **argv) {
    int o;
    unsigned i;
    int dumpconf = 0;
    static Handler handler[] = {
	/*  Argument	     Nr.args  Needconf  Handler function  */
	/*		   -1=no check				  */
	{ "services",		0,	1,	cmd_services	},
	{ "status",	       -1,	1,	cmd_status	},
	{ "stop",		0,	1,	cmd_stop	},
	{ "start", 		0,	1,	cmd_start	},
	{ "restart",		0,	1,	cmd_restart 	},
	{ "tell", 		4,	1,	cmd_tell 	},
	{ "configtest",		0,	1,	cmd_configtest 	},
    };

    /* Remember original ac/av/ep */
    org_argc = argc;
    org_argv = argv;

    /* Parse the command line.
     * NOTE: When changing the options list, remember to update:
     * - src/lib/usage.txt
     * - doc/main/using.yo
     * - doc/man/crossroads.yo
     */
    while ( (o = getopt (argc, argv, "bB:?c:fhvVal:stCm:xd:X:pP")) > 0 )
	switch (o) {
	case 'a':
	    opt.log_activity++;
	    break;
	case 'b':
	    dumpconf++;
	    break;
	case 'B':
	    opt.tcp_bufsz = atoi (optarg);
	    break;
	case 'c':
	    config_file = optarg;
	    break;
	case 'd':
	    opt.dns_cachettl = atoi (optarg);
	    break;
	case 'v':
	    flag_verbose++;
	    break;
	case 'l':
	    opt.log_facility = atoi(optarg);
	    break;
	case 'm':
	    opt.shmperm = strtol (optarg, 0, 8);
	    break;
	case 's':
	    opt.sloppyportbind++;
	    break;
	case 't':
	    tabular_status++;
	    break;
	case 'x':
	    xml_status++;
	    break;
	case 'X':
	    xml_xslt = optarg;
	    break;
	case 'C':
	    show_config();
	    exit (0);
	case 'p':
	    opt.leave_proctitle++;
	    break;
	case 'P':
	    parser_debug++;
	    break;
	case 'V':
	    puts (VER);
	    exit (0);
	case '?':
	case 'h':
	default:
	    usage ();
	}    

    /* Request to dump binary configuration? */
    if (dumpconf) {
	msg ("Parsing configuration %s", config_file);
	parser_open (config_file);
	parser_run();
	parser_close();
	options_write (1);
	config_write (1);
	exit (0);	    
    }
    
    /* We need at last one argument: the action */
    if (optind >= argc)
	usage();

    /* More sanity checks */
    if (xml_status && tabular_status)
	error ("Flags -t and -x are mutually exclusive");

    /* Act on the argument. */
    for (i = 0; i < sizeof(handler) / sizeof(Handler); i++) {
	/* Match the handler action string with argv, and match the
	 * required remaining arguments. If they match..
	 * scan the configuration (if needed) and GO!
	 */
	if (!strcmp (argv[optind], handler[i].arg) &&
	    (handler[i].narg == -1 ||
	     handler[i].narg == argc - optind - 1)) {
	    if (handler[i].needconf) {
		parser_open (config_file);
		parser_run();
		parser_close();
	    }
	    if (!handler[i].handler (argc - optind, argv + optind + 1))
		error ("'%s' failed", argv[optind]);
	    return (0);
	}
    }

    usage();
    return (1);
}
