/*************************************************************************
 * This file is part of Crosroads 1.76, a load balancer and fail over
 * utility for TCP. Copyright (c) Karel Kubat, distributed under GPL.
 * Visit http://crossroads.e-tunity.com for information.
 *************************************************************************/

#include "../crossroads.h"

static void wr (int fd, void const *buf, unsigned len) {
    int res;
    int towrite = (int)len;
    
    if ( (res = write (fd, buf, len)) != towrite )
	 error ("Failed to transmit %d bytes (%d)", towrite, res);
}

static void wr_int (int fd, int x) {
    wr (fd, &x, sizeof(int));
}

static void wr_uns (int fd, unsigned x) {
    wr (fd, &x, sizeof(unsigned));
}

static void wr_str (int fd, char const *s) {
    if (!s || !*s)
	wr_int (fd, 0);
    else {
	wr_int (fd, strlen(s));
	wr (fd, s, strlen(s));
    }
}

void config_write (int fd) {
    int i, j, k;

    msg ("Writing binary service definition to fd %d", fd);
    wr_int (fd, nservice);
    for (i = 0; i < nservice; i++) {	
	/* Basic stuff */
	wr_str (fd, service[i].name);
	wr_str (fd, service[i].bind);
	wr_uns (fd, service[i].port);
	wr_uns (fd, service[i].verbosity);
	wr_int (fd, (int)service[i].dispatchtype);
	wr_uns (fd, service[i].dispatchover);
	wr_str (fd, service[i].dispatchext);
	wr_uns (fd, service[i].rev_interval);
	wr_uns (fd, service[i].check_interval);
	wr_str (fd, service[i].check_cmd);
	wr_uns (fd, service[i].backlog);
	wr_uns (fd, service[i].shmkey);
	wr_uns (fd, service[i].connectiontimeout);
	wr_uns (fd, service[i].maxconnections);
	wr_int (fd, (int)service[i].type);
	wr_str (fd, service[i].allowfile);
	wr_str (fd, service[i].denyfile);
	wr_int (fd, service[i].uid);
	wr_int (fd, service[i].gid);
	wr_int (fd, service[i].inspection);

	/* Deny chain */
	wr_int (fd, service[i].ndenychain);
	for (j = 0; j < service[i].ndenychain; j++) {
	    wr_uns (fd, service[i].denychain[j].ip);
	    wr_uns (fd, service[i].denychain[j].mask);
	}

	/* Allow chain */
	wr_int (fd, service[i].nallowchain);
	for (j = 0; j < service[i].nallowchain; j++) {
	    wr_uns (fd, service[i].allowchain[j].ip);
	    wr_uns (fd, service[i].allowchain[j].mask);
	}

	/* Back ends */
	wr_int (fd, service[i].nbackend);
	for (j = 0; j < service[i].nbackend; j++) {
	    wr_str (fd, service[i].backend[j].name);
	    wr_int (fd, service[i].backend[j].initial_port);
	    wr_str (fd, service[i].backend[j].initial_server);
	    wr_int (fd, service[i].backend[j].verbosity);
	    wr_int (fd, service[i].backend[j].weight);
	    wr_int (fd, service[i].backend[j].decay);
	    wr_str (fd, service[i].backend[j].onstart);
	    wr_str (fd, service[i].backend[j].onend);
	    wr_str (fd, service[i].backend[j].dumpfile);
	    wr_str (fd, service[i].backend[j].thruputfile);
	    wr_str (fd, service[i].backend[j].timinglog);
	    wr_uns (fd, service[i].backend[j].maxconnections);
	    wr_str (fd, service[i].backend[j].stickycookie);
	    wr_int (fd, service[i].backend[j].retries);
	    wr_int (fd, service[i].backend[j].initial_avail);

	    wr_int (fd, service[i].backend[j].naddclientheader);
	    for (k = 0; k < service[i].backend[j].naddclientheader; k++)
		wr_str (fd, service[i].backend[j].addclientheader[k]);
	    wr_int (fd, service[i].backend[j].nsetclientheader);
	    for (k = 0; k < service[i].backend[j].nsetclientheader; k++)
		wr_str (fd, service[i].backend[j].setclientheader[k]);
	    wr_int (fd, service[i].backend[j].nappendclientheader);
	    for (k = 0; k < service[i].backend[j].nappendclientheader; k++)
		wr_str (fd, service[i].backend[j].appendclientheader[k]);

	    wr_int (fd, service[i].backend[j].naddserverheader);
	    for (k = 0; k < service[i].backend[j].naddserverheader; k++)
		wr_str (fd, service[i].backend[j].addserverheader[k]);
	    wr_int (fd, service[i].backend[j].nsetserverheader);
	    for (k = 0; k < service[i].backend[j].nsetserverheader; k++)
		wr_str (fd, service[i].backend[j].setserverheader[k]);
	    wr_int (fd, service[i].backend[j].nappendserverheader);
	    for (k = 0; k < service[i].backend[j].nappendserverheader; k++)
		wr_str (fd, service[i].backend[j].appendserverheader[k]);
	}
    }
}
