/*************************************************************************
 * This file is part of Crosroads 1.76, a load balancer and fail over
 * utility for TCP. Copyright (c) Karel Kubat, distributed under GPL.
 * Visit http://crossroads.e-tunity.com for information.
 *************************************************************************/
#include "../crossroads.h"

int ipf_loadfile (char const *fname, IpFilter **chain, int *nchain) {
    FILE *f;
    char buf[80];
    int nloaded = 0, ret = 0;
    IpFilter filter;

    /* No file? No update. */
    if (!fname || !*fname)
	return (0);

    msg ("Service %s: loading IP filter file: '%s'",
	 activeservice->name, fname);

    if (! (f = fopen (fname, "r")) ) {
	warning ("Service %s: can't read filter file '%s': %s",
		 activeservice->name, fname, strerror(errno));
	return (0);
    }

    free (*chain);
    *chain = 0;
    *nchain = 0;

    while (1) {
	if (fscanf (f, " %79s ", buf) < 1)
	    break;
	if (ipf_parse (buf, &filter)) {
	    warning ("Service %s: bad ip filter in file '%s'",
		     activeservice->name, fname);
	    ret++;
	} else {
	    msg ("Service %s: got ip filter specifier '%s'",
		 activeservice->name, buf);
	    *chain = xrealloc (*chain, (*nchain + 1) * sizeof(IpFilter));
	    (*chain)[*nchain] = filter;
	    (*nchain)++;
	    nloaded++;
	}
    }

    msg ("Service %s: loaded IP filter file '%s': %d successfully loaded",
	 activeservice->name, fname, nloaded);

    fclose (f);
    return (ret);
}
