/*************************************************************************
 * This file is part of Crosroads 1.76, a load balancer and fail over
 * utility for TCP. Copyright (c) Karel Kubat, distributed under GPL.
 * Visit http://crossroads.e-tunity.com for information.
 *************************************************************************/

#include "../crossroads.h"

/* To see the command that runs, even in the wakeup handler, undefine: */
/* #define DEBUG */

int sysrun (char const *cmd) {
    char *expanded;
    int ret;
    
    if (!cmd || ! *cmd)
	return (0);

    expanded = str_expand_format (cmd);
    
#   ifdef DEBUG
    msg ("Service %s: running command '%s'", activeservice->name, expanded);
#   else    
    if (program_stage != stage_retrying)
	msg ("Service %s: running command: '%s'",
	     activeservice->name, expanded);
#   endif
    
    uid_assume();
    ret = system (expanded);
    uid_restore();

    if (ret > 0)
	warning ("Service %s: command '%s' returned %d",
		 activeservice->name, expanded, ret);
    else if (ret < 0)
	warning ("Service %s: failed to run command '%s': %s",
		 activeservice->name, expanded, strerror(errno));
    
#   ifdef DEBUG
    if (!ret)
	msg ("Service %s: command '%s' returned %d",
	     activeservice->name, expanded, ret);
#    endif    

    free (expanded);

    return (ret);
}
